package com.loit.web.file.controller;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.loit.common.config.FastDFSConfigInfo;
import com.loit.common.json.AjaxJson;
import com.loit.common.json.LoitStatusCode;
import com.loit.modules.fastdfs.ErrorCode;
import com.loit.modules.fastdfs.FastDFSClient;
import com.loit.modules.fastdfs.FastDFSException;
import com.loit.modules.fastdfs.FileResponseData;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;

/**
 * @description 文件上传下载管理Controller
 * @author fangshiquan
 * @date 2019-11-08
 * @version V1.0
 */
@RestController
@RequestMapping(value = "api/file")
@Api(tags = "文件管理",  value = "FileRestController")
public class FileRestController {

	@Autowired
	private FastDFSConfigInfo fastDFSConfigInfo;
	private FastDFSClient fastDFSClient = new FastDFSClient();
	
	@GetMapping(value = "demo")
	public String demo(HttpServletRequest request) {
		System.out.println("dedmo");
		System.out.println(request.getHeader("a")); 
		return "demo";
	}
	
	/**
	 * @description 上传文件
	 * @param file 文件参数
	 * @param request 请求
	 * @return AjaxJson
	 */
	@PostMapping(value = "upload",consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @ApiOperation(value="文件上传", notes="文件上传")
	public AjaxJson fileUpload(@RequestParam(value = "file", required = true) MultipartFile file, HttpServletRequest request) {
		AjaxJson json = new AjaxJson();
		//文件保存
		FileResponseData fileResponseData =uploadSample(file, request);
		json.setSuccess(fileResponseData.isSuccess());
		json.setMsg(fileResponseData.getMessage());
		json.setErrorCode(fileResponseData.getCode());
		json.put("fileResponseData", fileResponseData);
		return json;
	}
	
	/**
     * 以附件形式下载文件
     *
     * @param filePath 文件地址
     * @param response
     */
    @RequestMapping(value="/download", method=RequestMethod.GET)
    @ApiImplicitParam(name = "filePath", value = "文件路径", required = true, dataType = "String",paramType = "query")
    @ApiOperation(value="文件下载", notes="文件下载")
    public void downloadFile(String filePath, HttpServletResponse response) throws FastDFSException {
        try {
        	if (filePath.startsWith("http://")) {
        		filePath =filePath.replace(fastDFSConfigInfo.getFileServerPrefix(), "");
			}
            fastDFSClient.downloadFile(filePath, response);
        } catch (FastDFSException e) {
            e.printStackTrace();
            throw e;
        }
    }
    
    

    /**
     * 根据指定的路径删除服务器文件，适用于没有保存数据库记录的文件
     *
     * @param filePath
     */
   
    @ApiImplicitParam(name = "filePath", value = "filePath", required = true, dataType = "String",paramType = "query")
    @ApiOperation(value="文件删除", notes="文件删除")
    @RequestMapping(value="/delete", method=RequestMethod.DELETE)
    public AjaxJson deleteFile(@RequestParam(value = "filePath") String filePath) {
    	AjaxJson json = new AjaxJson();
    	FileResponseData responseData = new FileResponseData();
        try {
        	if (filePath.startsWith("http://")) {
        		filePath =filePath.replace(fastDFSConfigInfo.getFileServerPrefix(), "");
			}
            fastDFSClient.deleteFile(filePath);
        } catch (FastDFSException e) {
            e.printStackTrace();
            responseData.setSuccess(false);
            responseData.setCode(e.getCode());
            responseData.setMessage(e.getMessage());
            json.setSuccess(false);
            json.setMsg(e.getMessage());
            json.setErrorCode(LoitStatusCode.FILE_DELETE_FAILED.statusCode);
    		json.put("fileResponseData", responseData);
        }
		return json;
    }
    
    
    
	  /**
     * 上传通用方法，只上传到服务器，不保存记录到数据库
     *
     * @param file
     * @param request
     * @return
     */
    @SuppressWarnings("static-access")
	public FileResponseData uploadSample(MultipartFile file, HttpServletRequest request){
        FileResponseData responseData = new FileResponseData();
        try {
            // 上传到服务器
        	String suffix =fastDFSClient.getFilenameSuffix(file.getOriginalFilename());
        	if(fastDFSClient.EXT_MAPS.get(suffix)!=null) {
        		 String filepath = fastDFSClient.uploadFileWithMultipart(file);
                 responseData.setFileName(file.getOriginalFilename());
                 responseData.setFilePath(filepath);
                 responseData.setFileType(FastDFSClient.getFilenameSuffix(file.getOriginalFilename()));
                 // 设置访文件的Http地址. 有时效性.
                /* String token = FastDFSClient.getToken(filepath, fastDFSHttpSecretKey);
                 responseData.setToken(token);*/
                 //responseData.setHttpUrl(fileServerAddr+"/"+filepath+"?"+token);
                 responseData.setHttpUrl(fastDFSConfigInfo.getFileServerPrefix()+"/"+filepath);	
        	}else {
        		responseData.setSuccess(false);
                responseData.setCode(ErrorCode.FILE_TYPE_ERROR_ALLOW.CODE);
                responseData.setMessage(ErrorCode.FILE_TYPE_ERROR_ALLOW.MESSAGE);
        	}
           
        } catch (FastDFSException e) {
            responseData.setSuccess(false);
            responseData.setCode(e.getCode());
            responseData.setMessage(e.getMessage());
        }

        return responseData;
    }
    
    /**
     * 并发接口测试
     * @return
     */
    @GetMapping("/getMapInfo")
    public Map<String,Object> getMapInfo(){
    	Map<String,Object> map=new HashMap<String, Object>();
    	map.put("name","fangshiquan");
    	map.put("age","20");
    	map.put("sex","男");
    	map.put("where","datian");
    	map.put("daxia","令狐冲");
    	map.put("daxia2","令狐冲ee");
    	map.put("da6xia2","333令狐冲ee");
    	map.put("da6hhh","33kk狐冲ee");
    	map.put("da6hhh2","33kk狐冲ee");
    	map.put("da6hhh1","33kk狐冲ee");
    	map.put("da6hhh3","33kk狐冲ee");
    	map.put("da6hhh4","33kk狐冲ee");
    	map.put("da6hhh5","33kk狐冲ee");
    	map.put("da6hhh6","33kk狐冲ee");
    	map.put("da6hhh7","33kk狐冲ee");
    	
    	map.put("da6哈哈ia2","333令狐冲ee");
    	map.put("d发给过hh","33kk狐冲ee");
    	map.put("da分为非h2","33kk狐冲ee");
    	map.put("da分为e非h1","33kk狐冲ee");
    	map.put("da6h发hh3","33kk狐冲ee");
    	map.put("da6hh分h4","33kk狐冲ee");
    	map.put("da6hh仍无法h5","33kk狐冲ee");
    	map.put("da6hh违法h6","33kk狐冲ee");
    	map.put("da6hhh7","33kk狐冲ee");
    	return map;
    }
}
